from Plugins.Plugin import PluginDescriptor
from Components.MultiContent import MultiContentEntryText
from Screens.Screen import Screen
from Components.ActionMap import ActionMap
from enigma import ePicLoad, addFont, ePythonMessagePump, eServiceReference, eTimer, gPixmapPtr, getDesktop, gFont, eListboxPythonMultiContent, RT_HALIGN_LEFT, RT_HALIGN_RIGHT, RT_HALIGN_CENTER, RT_VALIGN_CENTER, RT_VALIGN_TOP
from Components.MenuList import MenuList
from Screens.MessageBox import MessageBox
from Components.Pixmap import Pixmap
from Components.Label import Label
from Components.Language import language
from Tools.Directories import resolveFilename, SCOPE_LANGUAGE, SCOPE_PLUGINS

import gettext
from os import environ

import json
import urllib2
from bs4 import BeautifulSoup
import xml.etree.ElementTree as ET
import os
from twisted.internet import threads, reactor
import subprocess
import threading
import platform


lang = language.getLanguage()
environ["LANGUAGE"] = lang[:2]
gettext.bindtextdomain("enigma2", resolveFilename(SCOPE_LANGUAGE))
gettext.textdomain("enigma2")
gettext.bindtextdomain("JellyfinDream", "%s%s" % (resolveFilename(SCOPE_PLUGINS), "Extensions/JellyfinDream/locale/"))


def _(txt):
    t = gettext.dgettext("JellyfinDream", txt)
    if t == txt:
        t = gettext.gettext(txt)
    return t


DESKTOPSIZE = getDesktop(0).size()
if DESKTOPSIZE.width() > 1280:
    skinFactor = 1
else:
    skinFactor = 1.5

URL = "https://repo.jellyfin.org/releases/server/linux/stable/"

VERSION = "1.0.0"
INFO = "Package: enigma2-plugin-extensions-jellyfindream\nVersion: " + VERSION + "\nDescription: Jellyfin-Server-Plugin\nMaintainer: murxer <support@boxpirates.to>"

Jellyfin_SPINNER_DIRECTORY = "/usr/lib/enigma2/python/Plugins/Extensions/JellyfinDream/images/spinner"


class Spinner:
    def __init__(self):
        # Spinner Timer
        self['BackgroundJellyfinSpinner'] = Pixmap()
        self['JellyfinSpinner'] = Pixmap()
        self['BackgroundJellyfinSpinner'].hide()
        self['JellyfinSpinner'].hide()
        self.JellyfinSpinner = eTimer()
        self.JellyfinSpinnerStatus = False
        self.JellyfinSpinnerTimer = 1
        self.JellyfinSpinner_conn = self.JellyfinSpinner.timeout.connect(self.loadJellyfinSpinner)

    def stopJellyfinSpinner(self):
        self.JellyfinSpinnerStatus = False

    def startJellyfinSpinner(self):
        self.JellyfinSpinnerStatus = True
        self.loadJellyfinSpinner()

    def loadJellyfinSpinner(self):
        if self.JellyfinSpinnerStatus:
            png = "%s/%s.png" % (Jellyfin_SPINNER_DIRECTORY, str(self.JellyfinSpinnerTimer))
            self.showJellyfinSpinner(png)
        else:
            self['JellyfinSpinner'].hide()
            self['BackgroundJellyfinSpinner'].hide()

    def showJellyfinSpinner(self, png):
        self['BackgroundJellyfinSpinner'].show()
        self['JellyfinSpinner'].instance.setPixmapFromFile(png)
        self['JellyfinSpinner'].show()
        if self.JellyfinSpinnerTimer is not 8:
            self.JellyfinSpinnerTimer += 1
        else:
            self.JellyfinSpinnerTimer = 1
        self.JellyfinSpinner.start(400, True)


class JellyfinDream(Screen, Spinner):
    addFont("/usr/lib/enigma2/python/Plugins/Extensions/JellyfinDream/font/OpenSans-Regular.ttf", "PD", 100, False)

    def __init__(self, session):
        if DESKTOPSIZE.width() >= 1920:
            self.skin = """<screen backgroundColor="#00000c26" flags="wfNoBorder" name="JellyfinDreamHome" position="center,center" size="1920,1080" title="JellyfinDream">
                           <widget backgroundColor="#00000c26" foregroundColor="#00ffffff" font="PD; 42" position="1760,20" size="120,50" render="Label" source="global.CurrentTime" transparent="0" zPosition="2" halign="right" valign="center">
                             <convert type="ClockToText">Default</convert>
                           </widget>
                           <ePixmap position="50,50" size="500,500" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/JellyfinDream/images/jellyfin_500x500.png" zPosition="1" />
                           <widget name="ServerInfo" position="700,100" size="1200,400" transparent="0" foregroundColor="#00ffffff" backgroundColor="#00000c26" zPosition="1" font="PD; 38" valign="top" halign="left"/>
                           <widget name="keyRed" position="0,1020" size="384,50" transparent="0" foregroundColor="#00ffffff" backgroundColor="#00000c26" zPosition="1" font="PD; 32" valign="center" halign="center"/>
                           <eLabel name="lineRed" position="0,1070" size="384,10" backgroundColor="#00ff0000" zPosition="1"/>
                           <widget name="keyGreen" position="384,1020" size="384,50" transparent="0" foregroundColor="#00ffffff" backgroundColor="#00000c26" zPosition="1" font="PD; 32" valign="center" halign="center"/>
                           <eLabel name="lineGreen" position="384,1070" size="384,10" backgroundColor="#0000790b" zPosition="1"/>
                           <widget name="keyYellow" position="768,1020" size="384,50" transparent="0" foregroundColor="#00ffffff" backgroundColor="#00000c26" zPosition="1" font="PD; 32" valign="center" halign="center"/>
                           <eLabel name="lineYellow" position="768,1070" size="384,10" backgroundColor="#00d2b308" zPosition="1"/>
                           <widget name="keyBlue" position="1152,1020" size="384,50" transparent="0" foregroundColor="#00ffffff" backgroundColor="#00000c26" zPosition="1" font="PD; 32" valign="center" halign="center"/>
                           <eLabel name="lineBlue" position="1152,1070" size="384,10" backgroundColor="#000051ff" zPosition="1"/>
                           <widget name="keyInfo" position="1536,1020" size="384,50" transparent="0" foregroundColor="#00ffffff" backgroundColor="#00000c26" zPosition="1" font="PD; 32" valign="center" halign="center"/>
                           <eLabel name="lineInfo" position="1536,1070" size="384,10" backgroundColor="#00616161" zPosition="1"/>                   
                           <widget name="BackgroundJellyfinSpinner" position="0,0" size="1920,1080" gradient="#20000000,#70000000,horizontal" zPosition="98" />
                           <widget name="JellyfinSpinner" position="925,505" size="70,70" zPosition="99" />
                           </screen>
                        """
        else:
            self.skin = """<screen backgroundColor="#00000c26" flags="wfNoBorder" name="JellyfinDreamHome" position="center,center" size="1280,720" title="JellyfinDream">
                           <widget backgroundColor="#00000c26" foregroundColor="#00ffffff" font="PD; 28" position="1173,13" size="80,33" render="Label" source="global.CurrentTime" transparent="0" zPosition="2" halign="right" valign="center">
                             <convert type="ClockToText">Default</convert>
                           </widget>
                           <ePixmap position="33,33" size="333,333" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/JellyfinDream/images/jellyfin_333x333.png" zPosition="1" />
                           <widget name="ServerInfo" position="466,66" size="800,266" transparent="0" foregroundColor="#00ffffff" backgroundColor="#00000c26" zPosition="1" font="PD; 25" valign="top" halign="left"/>
                           <widget name="keyRed" position="0,680" size="256,33" transparent="0" foregroundColor="#00ffffff" backgroundColor="#00000c26" zPosition="1" font="PD; 21" valign="center" halign="center"/>
                           <eLabel name="lineRed" position="0,713" size="256,6" backgroundColor="#00ff0000" zPosition="1"/>
                           <widget name="keyGreen" position="256,680" size="256,33" transparent="0" foregroundColor="#00ffffff" backgroundColor="#00000c26" zPosition="1" font="PD; 21" valign="center" halign="center"/>
                           <eLabel name="lineGreen" position="256,713" size="256,6" backgroundColor="#0000790b" zPosition="1"/>
                           <widget name="keyYellow" position="512,680" size="256,33" transparent="0" foregroundColor="#00ffffff" backgroundColor="#00000c26" zPosition="1" font="PD; 21" valign="center" halign="center"/>
                           <eLabel name="lineYellow" position="512,713" size="256,6" backgroundColor="#00d2b308" zPosition="1"/>
                           <widget name="keyBlue" position="768,680" size="256,33" transparent="0" foregroundColor="#00ffffff" backgroundColor="#00000c26" zPosition="1" font="PD; 21" valign="center" halign="center"/>
                           <eLabel name="lineBlue" position="768,713" size="256,6" backgroundColor="#000051ff" zPosition="1"/>
                           <widget name="keyInfo" position="1024,680" size="256,33" transparent="0" foregroundColor="#00ffffff" backgroundColor="#00000c26" zPosition="1" font="PD; 21" valign="center" halign="center"/>
                           <eLabel name="lineInfo" position="1024,713" size="256,6" backgroundColor="#00616161" zPosition="1"/>
                           <widget name="BackgroundJellyfinSpinner" position="0,0" size="1280,720" gradient="#20000000,#70000000,horizontal" zPosition="98" />
                           <widget name="JellyfinSpinner" position="616,336" size="46,46" zPosition="99" />
                           </screen>
                        """
        Screen.__init__(self, session)

        Spinner.__init__(self)

        self['actions'] = ActionMap(['JellyfinDream_Actions'],
                                    {'cancel': self.keyChancel,
                                     'info': self.keyInfo,
                                     'red': self.keyRed,
                                     'green': self.keyGreen,
                                     'yellow': self.keyYellow,
                                     'blue': self.keyBlue,
                                     '0': self.close
                                     }, -1)

        self["ServerInfo"] = Label("")
        self["keyRed"] = Label("")
        self["keyGreen"] = Label("")
        self["keyYellow"] = Label("")
        self["keyBlue"] = Label(_("Server downloader"))
        self["keyInfo"] = Label("Info")
        self.data_list = []

        self.onLayoutFinish.append(self.createSetup)

    def createSetup(self):
        if os.path.isdir("/data/jellyfin-server"):
            txt = "Jellyfin-server: %s\n" % _("installed")
            self["keyGreen"].setText("Start/Stop")
        else:
            txt = "Jellyfin-server: %s\n" % _("not installed")
            self["keyGreen"].setText("")
        if os.path.isdir("/data/jellyfin-server_backup"):
            self["keyRed"].setText(_("Restore Backup"))
        else:
            self["keyRed"].setText("")
        command = Command("systemctl status jellyfin")
        command.run(timeout=3)
        cmd_out = command.out
        if "Active: active" in cmd_out:
            txt += "Jellyfin: %s\n" % _("active")
        else:
            txt += "Jellyfin: %s\n" % _("inactive")
        if "enabled;" in cmd_out:
            txt += "Jellyfin autostart: %s\n" % _("enabled")
            self["keyYellow"].setText(_("Disable Autostart"))
        else:
            txt += "Jellyfin autostart: %s\n" % _("deactivated")
            self["keyYellow"].setText(_("Enable Autostart"))
        self["ServerInfo"].setText(txt)

    def setServerList(self):
        self.data_list = []
        if not self.JellyfinSpinnerStatus:
            get_server_list(self.backSetServerList)

    def backSetServerList(self, data):
        self.stopJellyfinSpinner()
        self.data_list = data

    def keyChancel(self):
        if not self.JellyfinSpinnerStatus:
            self.close()

    def backDownload(self, callback):
        self.stopJellyfinSpinner()
        self.createSetup()
        self.session.open(MessageBox, windowTitle=_("JellyfinDream Info"), text=callback, type=MessageBox.TYPE_INFO)

    def keyRed(self):
        if not self.JellyfinSpinnerStatus:
            if os.path.isdir("/data/jellyfin-server_backup"):
                self.session.openWithCallback(self.backRestoreBackup, MessageBox, _("Restore Backup?"), MessageBox.TYPE_YESNO, default=False)

    def backRestoreBackup(self, answer):
        if answer:
            if statusJellyfin():
                stopJellyfin()
            if os.path.isdir("/data/jellyfin-server"):
                os.system("rm -R /data/jellyfin-server")
            os.system("mv /data/jellyfin-server_backup /data/jellyfin-server")
            self.session.openWithCallback(self.backRestoreBackupStart, MessageBox, _("Backup has been activated.\nShould jellyfin be started again?"), MessageBox.TYPE_YESNO, default=True)

    def backRestoreBackupStart(self, answer):
        if answer:
            startJellyfin()
        self.createSetup()

    def keyGreen(self):
        if not self.JellyfinSpinnerStatus:
            if statusJellyfin():
                stopJellyfin()
                self.session.open(MessageBox, windowTitle=_("JellyfinDream Info"), text=_("Jellyfin-Server stopped."), type=MessageBox.TYPE_INFO)
            else:
                startJellyfin()
                self.session.open(MessageBox, windowTitle=_("JellyfinDream Info"), text=_("Jellyfin-Server started."), type=MessageBox.TYPE_INFO)
            self.createSetup()

    def keyYellow(self):
        if not self.JellyfinSpinnerStatus:
            if statusJellyfinAutostart():
                disableJellyfinAutostart()
                self.session.open(MessageBox, windowTitle=_("JellyfinDream Info"), text=_("Autostart disabled."), type=MessageBox.TYPE_INFO)
            else:
                enabledJellyfinAutostart()
                self.session.open(MessageBox, windowTitle=_("JellyfinDream Info"), text=_("Autostart enabled."), type=MessageBox.TYPE_INFO)
            self.createSetup()

    def keyBlue(self):
        if not self.JellyfinSpinnerStatus:
            got_server_list(self.backServerList)

    def backServerList(self, callback):
        self.stopJellyfinSpinner()
        if callback:
            self.session.openWithCallback(self.backServerDownloader, JellyfinDreamDownloader, callback)
        else:
            self.session.open(MessageBox, windowTitle=_("JellyfinDream Info"), text=_("Server lists download error!"), type=MessageBox.TYPE_ERROR)

    def backServerDownloader(self):
        self.createSetup()

    def keyInfo(self):
        txt = INFO
        if statusJellyfin():
            info = getSystemInfoPublic()
            if info:
                txt = "Server-name: %s\n" % info.get("ServerName", "N/A").encode("utf-8")
                txt += "Server-version: %s\n" % info.get("Version", "N/A").encode("utf-8")
                txt += "Local-address: %s" % info.get("LocalAddress", "N/A").encode("utf-8")
        self.session.open(MessageBox, windowTitle=_("JellyfinDream Info"), text=txt, type=MessageBox.TYPE_INFO)


class JellyfinDreamDownloader(Screen, Spinner):
    addFont("/usr/lib/enigma2/python/Plugins/Extensions/JellyfinDream/font/OpenSans-Regular.ttf", "PD", 100, False)

    def __init__(self, session, data):
        if DESKTOPSIZE.width() >= 1920:
            self.skin = """<screen backgroundColor="#00000c26" flags="wfNoBorder" name="JellyfinDreamDownloader" position="center,center" size="1920,1080" title="JellyfinDreamDownloader">
                           <widget backgroundColor="#00000c26" foregroundColor="#00ffffff" font="PD; 42" position="1760,20" size="120,50" render="Label" source="global.CurrentTime" transparent="0" zPosition="2" halign="right" valign="center">
                             <convert type="ClockToText">Default</convert>
                           </widget>
                           <ePixmap position="50,50" size="500,500" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/JellyfinDream/images/jellyfin_500x500.png" zPosition="1" />
                           <widget name="MenuList" position="700,100" size="1200,200" transparent="0" foregroundColor="#00ffffff" backgroundColor="#00000c26" zPosition="1"/>
                           <eLabel name="lineRed" position="0,1070" size="384,10" backgroundColor="#00ff0000" zPosition="1"/>
                           <eLabel name="lineGreen" position="384,1070" size="384,10" backgroundColor="#0000790b" zPosition="1"/>
                           <eLabel name="lineYellow" position="768,1070" size="384,10" backgroundColor="#00d2b308" zPosition="1"/>
                           <eLabel name="lineBlue" position="1152,1070" size="384,10" backgroundColor="#000051ff" zPosition="1"/>
                           <eLabel name="lineInfo" position="1536,1070" size="384,10" backgroundColor="#00616161" zPosition="1"/>                   
                           <widget name="BackgroundJellyfinSpinner" position="0,0" size="1920,1080" gradient="#20000000,#70000000,horizontal" zPosition="98" />
                           <widget name="JellyfinSpinner" position="925,505" size="70,70" zPosition="99" />
                           </screen>
                        """
        else:
            self.skin = """<screen backgroundColor="#00000c26" flags="wfNoBorder" name="JellyfinDreamDownloader" position="center,center" size="1280,720" title="JellyfinDreamDownloader">
                           <widget backgroundColor="#00000c26" foregroundColor="#00ffffff" font="PD; 28" position="1173,13" size="80,33" render="Label" source="global.CurrentTime" transparent="0" zPosition="2" halign="right" valign="center">
                             <convert type="ClockToText">Default</convert>
                           </widget>
                           <ePixmap position="33,33" size="333,333" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/JellyfinDream/images/jellyfin_333x333.png" zPosition="1" />
                           <widget name="MenuList" position="466,66" size="800,133" transparent="0" foregroundColor="#00ffffff" backgroundColor="#00000c26" zPosition="1"/>
                           <eLabel name="lineRed" position="0,713" size="256,6" backgroundColor="#00ff0000" zPosition="1"/>
                           <eLabel name="lineGreen" position="256,713" size="256,6" backgroundColor="#0000790b" zPosition="1"/>
                           <eLabel name="lineYellow" position="512,713" size="256,6" backgroundColor="#00d2b308" zPosition="1"/>
                           <eLabel name="lineBlue" position="768,713" size="256,6" backgroundColor="#000051ff" zPosition="1"/>
                           <eLabel name="lineInfo" position="1024,713" size="256,6" backgroundColor="#00616161" zPosition="1"/>
                           <widget name="BackgroundJellyfinSpinner" position="0,0" size="1280,720" gradient="#20000000,#70000000,horizontal" zPosition="98" />
                           <widget name="JellyfinSpinner" position="616,336" size="46,46" zPosition="99" />
                           </screen>
                        """
        Screen.__init__(self, session)

        Spinner.__init__(self)

        self['actions'] = ActionMap(['JellyfinDream_Actions'],
                                    {'ok': self.keyOk,
                                     'cancel': self.keyChancel,
                                     '0': self.close
                                     }, -1)

        self["MenuList"] = Label("")
        self.chooseMenu = MenuList([], enableWrapAround=True, content=eListboxPythonMultiContent)
        self.chooseMenu.l.setFont(0, gFont('PD', int(32 / skinFactor)))
        self.chooseMenu.l.setItemHeight(int(50 / skinFactor))
        self["MenuList"] = self.chooseMenu
        self.data_list = data

        self.onLayoutFinish.append(self.createSetup)

    def createSetup(self):
        self.chooseMenu.setList(map(menu_entry, self.data_list))

    def keyChancel(self, callback=None):
        if not self.JellyfinSpinnerStatus:
            self.close()

    def keyOk(self):
        if not self.JellyfinSpinnerStatus:
            self.session.openWithCallback(self.backDownloadMessage, MessageBox, _("Install server?"), MessageBox.TYPE_YESNO, default=True)

    def backDownloadMessage(self, answer):
        if answer:
            self.startJellyfinSpinner()
            if statusJellyfin():
                stopJellyfin()
            url = self["MenuList"].getCurrent()[0][2]
            get_download_data(url, self.backDownload)

    def backDownload(self, callback):
        self.stopJellyfinSpinner()
        self.session.openWithCallback(self.keyChancel, MessageBox, windowTitle=_("JellyfinDream Info"), text=callback, type=MessageBox.TYPE_INFO)


def menu_entry(entry):
    res = [entry]
    res.append(MultiContentEntryText(pos=(int(5 / skinFactor), 0),
                                     size=(int(700 / skinFactor), int(50 / skinFactor)),
                                     font=0,
                                     flags=RT_HALIGN_LEFT | RT_VALIGN_CENTER,
                                     text=entry[0]))
    return res


# to get consolen-output with timeout-function
class Command(object):
    def __init__(self, cmd):
        self.cmd = cmd
        self.process = None
        self.out = ""
        self.err = ""

    def run(self, timeout):
        def target():
            self.process = subprocess.Popen(self.cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            (self.out, self.err) = self.process.communicate()

        thread = threading.Thread(target=target)
        thread.start()
        self.timeout = False

        thread.join(float(timeout))
        if thread.is_alive():
            self.process.terminate()
            self.timeout = True
            self.process = None
            thread = None
            return


arch_list = {"aarch64": "_arm64.tar.gz",
             "arm64": "_arm64.tar.gz",
             "armv7l": "_armhf.tar.gz",
             "armhf": "_armhf.tar.gz",
             "default": "default"}


def getProcessorArchitecturePlatform():
    architecture = "default"
    try:
        architecture = platform.machine()
    except Exception as error:
        print(error)
    print(architecture)
    return arch_list[architecture] if arch_list.get(architecture) else "default"


def disableJellyfinAutostart():
    command = Command("systemctl disable jellyfin")
    command.run(timeout=3)


def enabledJellyfinAutostart():
    command = Command("systemctl enable jellyfin")
    command.run(timeout=3)


def statusJellyfinAutostart():
    enabled = False
    command = Command("systemctl status jellyfin")
    command.run(timeout=3)
    cmd_out = command.out
    if "enabled;" in cmd_out:
        enabled = True
    return enabled


def statusJellyfin():
    isRunning = False
    command = Command("systemctl status jellyfin")
    command.run(timeout=3)
    cmd_out = command.out
    if "Active: active" in cmd_out:
        isRunning = True
    return isRunning


def startJellyfin():
    command = Command("systemctl start jellyfin")
    command.run(timeout=3)


def stopJellyfin():
    command = Command("systemctl stop jellyfin")
    command.run(timeout=3)


def get_server_list(callback):
    threads.deferToThread(got_server_list, callback)


def got_server_list(callback):
    data = []
    soup = get_soup_response(URL)
    if soup:
        server_data = soup.find_all("tbody")
        try:
            if server_data and len(server_data) >= 2:
                for tar in server_data[1].find_all("a", href=True):
                    url = URL + tar.get_text().strip().encode("utf-8") if tar.get_text() else ""
                    if url and getProcessorArchitecturePlatform() in url and "combined/jellyfin" in url and "sha256sum" not in url and "musl" not in url:
                        name = url.split("/")[-1]
                        version = url.split("_")[1]
                        data.append((name, version, url))
        except Exception as error:
            print("[JellyfinDream]: getServerList error: %s" % str(error))
    reactor.callFromThread(callback, data)


def get_soup_response(url):
    soup = ""
    try:
        response = urllib2.urlopen(url, timeout=8)
        data = response.read()
        soup = BeautifulSoup(data, 'html.parser')
        # handle errors
    except urllib2.HTTPError as e:
        print("[JellyfinDream] HTTP error: %s %s" % (e.code, url))
    except urllib2.URLError as e:
        print("[JellyfinDream] URL error: %s %s" % (e.reason, url))
    return soup


def get_download_data(url, callback):
    threads.deferToThread(got_download_data, url, callback)


def got_download_data(url, callback):
    try:
        f = urllib2.urlopen(url, timeout=8)
        # Open our local file for writing
        destination = "/data/%s" % os.path.basename(url)
        with open(destination, "wb") as local_file:
            local_file.write(f.read())
    # handle errors
    except urllib2.HTTPError as e:
        print("[JellyfinDream] HTTP error: %s %s" % (e.code, url))
        result = str("HTTP Error: %s %s" % (e.code, url))
    except urllib2.URLError as e:
        print("[JellyfinDream] URL error: %s %s" % (e.reason, url))
        result = str("URL Error: %s %s" % (e.reason, url))
    else:
        if os.path.isdir("/data/jellyfin-server"):
            if os.path.isdir("/data/jellyfin-server_backup"):
                os.system("rm -R /data/jellyfin-server_backup")
            os.system("mv '/data/jellyfin-server' '/data/jellyfin-server_backup'")
        os.system("tar xfvz '%s' -C '/data'" % destination)
        destination_server = "/data/%s" % os.path.basename(url).replace(getProcessorArchitecturePlatform(), "")
        if os.path.isdir(destination_server):
            os.system("mv '%s' '/data/jellyfin-server'" % destination_server)
        os.system("rm '%s'" % destination)
        result = _("Jellyfin-server successfully installed!")
    reactor.callFromThread(callback, result)


def getSystemInfoPublic():
    try:
        root = ET.parse('/root/.config/jellyfin/system.xml').getroot()
        port = root.find("PublicPort").text
    except Exception as error:
        print("[JellyfinDream] xml error: %s" % str(error))
        port = 8096
    data = {}
    try:
        url = "http://127.0.0.1:{port}/System/Info/Public".format(port=port)
        response = urllib2.urlopen(url, timeout=3)
        data = json.load(response)
    except Exception as error:
        print("[JellyfinDream] getSystemInfoPublic error: %s" % str(error))
    return data


def main(session, **kwargs):
    session.open(JellyfinDream)


def Plugins(path, **kwwargs):
    return [
        PluginDescriptor(name=_('Jellyfin Server'), description=_('Jellyfin-Server-Plugin v%s' % VERSION),
                         where=[PluginDescriptor.WHERE_PLUGINMENU, PluginDescriptor.WHERE_EXTENSIONSMENU], fnc=main,
                         icon='plugin.png')
    ]
